/*
 * Created on Jun 13, 2003
 */
package core;

import java.security.Key;
import java.security.NoSuchAlgorithmException;

/**
 * This method represents a Message which is tagged
 * @author Xavier Didelot
 */
public class MessageT extends Message {
	
	/**
	 * Returns the tag of this Message
	 * @return the tag of this Message
	 * @throws Exception if the message does not contain a valid tag
	 */
	public static Tag getTag(Message m) throws Exception {
		return ((MessageT)m).getTag();
	}

	public Tag getTag() throws Exception {
		IMessage[] fields=getAllFields();
		String tag="";
		for (int i=0;i<fields.length;i++)
			tag=tag+((MessageT)fields[i]).getOneTag();
		return new Tag(tag);
	}
	
	private String getOneTag() throws Exception {
		String str = getString();
		int coma=str.indexOf(',');
		if (coma == -1) throw new Exception("No tag found in "+str);
		return str.substring(0, coma);
	}
	
	public String getText() throws Exception {
		String str = getString();
		int coma=str.indexOf(',');
		if (coma == -1) throw new Exception("No tag found");
		return new String(str.substring(coma+1, str.length()));	
	}

/*	public void addMessage(Message mmsg) {
		if (this.getString()=="") {setString(mmsg.getString());return;}
		try {
			MessageT msg=(MessageT)mmsg;
			String t=getTag().getCode();
			String t2=msg.getTag().getCode();
			setString(t+","+getText()+"@"+t2+msg.getText());
		} catch (Exception e) {e.printStackTrace();}
	}*/
	
	public void decrypt(String algo, Key key) throws Exception {
		setString(getText());
		super.decrypt(algo,key);
	}	
	
	public void encrypt(String algo, Key key) throws Exception {
		Tag t=getTag();
		if (key.getClass().getName().endsWith("PublicKey")) t.encPub(); else
		if (key.getClass().getName().endsWith("PrivateKey")) t.encSec(); else
		t.encKey();
		super.encrypt(algo,key);
		setString(t.getCode()+","+getString());
	}

	public void hash(String method) throws NoSuchAlgorithmException {
		try {
			super.setString(getText());
			super.hash(method);
			setString("H,"+getString());
		} catch (Exception e) {e.printStackTrace();}
	}
		
	public void setString(String str) {
		super.setString(str);
		/*try {
			getTag();
		} catch (Exception e) {System.out.println("Untaged Message!");}		*/
	}

	public MessageT() {
		super();
	}

	public MessageT(Agent agent) {
		super(agent);
		setString("A,"+getString());
	}

	public MessageT(INonce nonce) {
		super(nonce);
		if (nonce.getClass().getName().compareTo("core.Nonce")==0) setString("N,"+getString());
	}

	public MessageT(Message m1, Message m2) {
		super(m1, m2);
	}

	public IMessage[] getAllFields() {
		Message m=new Message();
		try {m.setString(getString());} catch (Exception e) {e.printStackTrace();}
		IMessage[] a=m.getAllFields();
		for (int i=0;i<a.length;i++) {MessageT mt=new MessageT();
			mt.setString(a[i].getString());
			a[i]=mt;}
		return a;
	}

	public void removeTags() throws Exception {
		IMessage[] fields=getAllFields();
		if (fields.length==0) return;
		String s="";
		for (int i=0;i<fields.length;i++)
			{
				MessageT mt=(MessageT)fields[i];
				if (mt.getOneTag().compareTo("A")==0) {
					//System.out.println("Removed A for "+mt.getText());
					mt.setString(mt.getText());
					} 
				s=s+"@"+mt.getString();
			}
		setString(s.substring(1));
	}

	public void setStringAddr(String string) {
		setString("D,"+string);		
	}

	public void unxor(IMessage message) {
		Message m1=new Message(),m2=new Message();
		try {
			m1.setString(getText());
			m2.setString(((MessageT)message).getText());
			m1.unxor(m2);
			setString("N,"+m1.getString());
		} catch (Exception e) {e.printStackTrace();}
	}

	public void xor(IMessage message) {
		Message m1=new Message(),m2=new Message();
		try {
			m1.setString(getText());
			m2.setString(((MessageT)message).getText());
			m1.xor(m2);
			setString("XN+N),"+m1.getString());
		} catch (Exception e) {e.printStackTrace();}
	}
	
	protected Object clone() throws CloneNotSupportedException {
		MessageT m=new MessageT();
		m.setString(this.getString());
		return m;
	}
	
}
