package protocols;

import core.*;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.lang.reflect.Array;
import java.io.ObjectOutputStream;
import java.io.ByteArrayOutputStream;
import java.math.BigInteger;

public class COMPANY extends Protocol {
   
   public COMPANY (ObjectOutputStream oos) {
      super("core.MessageT","core.MessageKeyT","core.NonceT");
      this.oos=oos;
      }
   
   public void run() {
      try {
         
         println("creating nonce answer");
         createNewNonce("answer");
         println("creating nonce price");
         createNewNonce("price");
         println("creating nonce receipt");
         createNewNonce("receipt");
         println("creating nonce end");
         createNewNonce("end");
         
         println("waiting for m1");
         Message m1 = message(get("A",4001));
         checkTag(m1,"APSA)N)");
         addParts(m1.getAllFields(),2);
         m0=getPart();
         setNameToId(m0.getString(),"A");
         m0=getPart();
         println("Decrypting m0");
         m0.decrypt("RSA", getPrivateKey(nameFromId("C")));
         checkTag(m0,"SA)N");
         addParts(m0.getAllFields(),2);
         m0=getPart();
         println("Decrypting m0");
         m0.decrypt("RSA", getPublicKey(nameFromId("A")));
         checkTag(m0,"A");
         addParts(m0.getAllFields(),1);
         m0=getPart();
         setNameToId(m0.getString(),"C");
         
         m0=getPart();
         setValueToNonce(m0.getString(),"request");
         
         
         
         println("Building message m2");
         Message m2 =message();
         m2.addMessage(message(nonceFromId("request")));
         m2.addMessage(message(nonceFromId("answer")));
         println("Encrypting m2");
         m2.encrypt("RSA", getPublicKey(nameFromId("A")));
         println("Sending message m2");
         put("A", m2.getString());
         
         println("waiting for m3");
         Message m3 = message(get("A"));
         checkTag(m3,"PNNSAD)SPN)))");
         println("Decrypting m3");
         m3.decrypt("RSA", getPrivateKey(nameFromId("C")));
         checkTag(m3,"NNSAD)SPN))");
         addParts(m3.getAllFields(),4);
         m0=getPart();
         setValueToNonce(m0.getString(),"answer");
         m0=getPart();
         setValueToNonce(m0.getString(),"choice");
         m0=getPart();
         println("Decrypting m0");
         m0.decrypt("RSA", getPublicKey(nameFromId("A")));
         checkTag(m0,"AD");
         addParts(m0.getAllFields(),2);
         m0=getPart();
         setNameToId(m0.getString(),"B");
         m0=getPart();
         setIPToId(m0.getString(),"B");
         
         m0=getPart();
         Message c=message(m0.getString());
         
         
         println("Building message m4");
         Message m4 =message();
         m4.addMessage(message(agentFromId("C")));
         m0=message();
         m0.addMessage(message(agentFromId("A")));
         storeM0();
         m0.addMessage(message(agentFromId("B")));
         println("Encrypting m0");
         m0.encrypt("RSA", getPrivateKey(nameFromId("C")));
         m0=message(restoreM0(),m0);
         m0.addMessage(message(nonceFromId("price")));
         println("Encrypting m0");
         m0.encrypt("RSA", getPublicKey(nameFromId("B")));
         m4.addMessage(m0);
         println("Sending message m4");
         put("B", m4.getString(),4000);
         
         println("waiting for m5");
         Message m5 = message(get("B"));
         checkTag(m5,"PNN)");
         println("Decrypting m5");
         m5.decrypt("RSA", getPrivateKey(nameFromId("C")));
         checkTag(m5,"NN");
         addParts(m5.getAllFields(),2);
         m0=getPart();
         setValueToNonce(m0.getString(),"price");
         m0=getPart();
         setValueToNonce(m0.getString(),"transactionID");
         
         
         println("Building message m6");
         Message m6 =message();
         m6.addMessage(message(nonceFromId("transactionID")));
         m0=message();
         m0.addMessage(c);
         m6.addMessage(m0);
         println("Encrypting m6");
         m6.encrypt("RSA", getPublicKey(nameFromId("B")));
         println("Sending message m6");
         put("B", m6.getString());
         
         println("waiting for m7");
         Message m7 = message(get("B"));
         checkTag(m7,"PSA)SPN))N)");
         println("Decrypting m7");
         m7.decrypt("RSA", getPrivateKey(nameFromId("C")));
         checkTag(m7,"SA)SPN))N");
         addParts(m7.getAllFields(),3);
         m0=getPart();
         println("Decrypting m0");
         m0.decrypt("RSA", getPublicKey(nameFromId("B")));
         checkTag(m0,"A");
         addParts(m0.getAllFields(),1);
         m0=getPart();
         setNameToId(m0.getString(),"C");
         
         m0=getPart();
         Message e=message(m0.getString());
         m0=getPart();
         setValueToNonce(m0.getString(),"summary");
         
         
         println("Building message m8");
         Message m8 =message();
         m0=message();
         m0.addMessage(message(agentFromId("A")));
         println("Encrypting m0");
         m0.encrypt("RSA", getPrivateKey(nameFromId("C")));
         m8.addMessage(m0);
         m0=message();
         m0.addMessage(e);
         m8.addMessage(m0);
         m8.addMessage(message(nonceFromId("receipt")));
         println("Encrypting m8");
         m8.encrypt("RSA", getPublicKey(nameFromId("A")));
         println("Sending message m8");
         put("A", m8.getString());
         
         println("waiting for m9");
         Message m9 = message(get("A"));
         checkTag(m9,"PNN)");
         println("Decrypting m9");
         m9.decrypt("RSA", getPrivateKey(nameFromId("C")));
         checkTag(m9,"NN");
         addParts(m9.getAllFields(),2);
         m0=getPart();
         setValueToNonce(m0.getString(),"receipt");
         m0=getPart();
         setValueToNonce(m0.getString(),"acknowledgement");
         
         
         println("Building message m10");
         Message m10 =message();
         m0=message();
         m0.addMessage(message(agentFromId("B")));
         println("Encrypting m0");
         m0.encrypt("RSA", getPrivateKey(nameFromId("C")));
         m10.addMessage(m0);
         m10.addMessage(message(nonceFromId("acknowledgement")));
         m10.addMessage(message(nonceFromId("end")));
         println("Encrypting m10");
         m10.encrypt("RSA", getPublicKey(nameFromId("B")));
         println("Sending message m10");
         put("B", m10.getString());
         
         writeObject(nameFromId("A"));
         writeObject(nonceFromId("choice"));
         writeObject(nonceFromId("price"));
         writeObject(nonceFromId("receipt"));
         
         
         } catch (Exception e) {e.printStackTrace();System.exit(0);}
      }
   
   public static void subMain(String[] args,ObjectOutputStream oos) {
      COMPANY C;
      while (true) {
         C=new COMPANY(oos);
         args=C.getArgs(args);
         C.newAgent("A",4001);
         C.start();
         }
      }
   
   public String[] getArgs(String[] args) {
      try {
         if (args.length != 3) {
            args = (String[]) Array.newInstance("".getClass(), 3);
            BufferedReader in2 =
            new BufferedReader(new InputStreamReader(System.in));
            println("Type the KeyStore file to use  [keystore]");
            args[0]=in2.readLine();
            if (args[0].compareTo("")==0) args[0]="keystore";
            println("Type the password to access it [duitama]");
            args[1]=in2.readLine();
            if (args[1].compareTo("")==0) args[1]="duitama";
            println("Type the name of agent C [C]");
            args[2]=in2.readLine();
            if (args[2].compareTo("")==0) args[2]="C";
            }
         setKS(args[0],args[1]);
         setNameToId(args[2],"C");
         } catch (Exception e) {e.printStackTrace();}
      return args;
      }
   
   public static void main(String[] args) {
      ObjectOutputStream oos=null;
      try {
         ByteArrayOutputStream os = new ByteArrayOutputStream();
         oos=new ObjectOutputStream(os);
         PrintOOS p=new PrintOOS("COMPANY",oos,os);
         p.start();}
      catch (Exception e) {e.printStackTrace();}
      subMain(args,oos);
      }
   }